"""
trex meta.py
Includes variables with meta info to import in other packages

CHANGE VALUES HERE WHEN UPDATED (VERSION)
"""

# App info
APP_NAME = "trex"
APP_VERSION = "0.1.4"

# Server info
TREX_SERVER = "trexapi.deta.dev"

# Config options
config_options = [
    "intro",
    "tips"
]

# "Getting to work" strings
working_strings = [
    "I'm on the job",
    "Let's roar!",
    "Roooaaar!",
    "Never seen a Templatosaurus Rex before?",
    "Getting to work",
    "I'm working, chill!",
    "So much to do today...",
    "How many times did you use this template again?",
    "I'm a Templatosaurus Rex",
    "As green as your terminal font, or not...",
    "I'm like a Genie, you only need one command to call me!"
]